#ifndef _FILE_SERVICES_H
#define _FILE_SERVICES_H

#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include "simpletools.h"
#include <dirent.h>
#include "config.h"

#define MIN_LCD_CUR_POS 0
#define MAX_LCD_CUR_POS 3

#define LCD_OFF 21 // Turn the display off
#define LCD_ON_CUR_OFF_NO_BLK 22 // Turn the display on, with cursor off and no blink
#define LCD_ON_CUR_OFF_BLK 23 // Turn the display on, with cursor off and character blink
#define LCD_ON_CUR_ON_NO_BLK 24 // Turn the display on, with cursor on and no blink (Default)
#define LCD_ON_CUR_ON_BLK 25 // Turn the display on, with cursor on and character blink

#define LCD_MV_CUR_LN_0_POS_0 128 // Move cursor to line 0, position 0
#define LCD_MV_CUR_LN_1_POS_0 148 // Move cursor to line 1, position 0
#define LCD_MV_CUR_LN_2_POS_0 168 // Move cursor to line 2, position 0
#define LCD_MV_CUR_LN_3_POS_0 188 // Move cursor to line 3, position 0

#define LCD_CLEAR 12
#define LCD_BACK_LT_ON 17
#define LCD_BACK_LT_OFF 18

#define LCD_ALERT_LENGTH 211 // Length - (208-214)
#define LCD_ALERT_SCALE 219 // Scale - (215-219)
#define LCD_ALERT_NOTE 221 // Note - (220-231)

#define LED_RED UI_RED_LED // LED - error indicator
#define LED_AMBER UI_AMBER_LED // LED - currently for testing, ultimately when file is opend for reading
#define LCD_RESET UI_EXIT_PB // Pushbutton - go to root diectory
#define LCD_PARENT UI_BACK_PB // Pushbutton - if not root, go to parent directory
#define LCD_CONFIRM UI_CONFIRM_PB // Pushbutton - confirm file or directory selection, and go 
#define LCD_SELECT UI_SELECT_PB // Pushbutton - select file or directory
#define LCD_SCR_DWN UI_SCR_DWN_PB // Pushbutton - scroll down display
#define LCD_SCR_UP UI_SCR_UP_PB // Pushbutton - scroll up display

#define SD_DO PMC_IO1_DO // SD data out
#define SD_CLK PMC_CLK // SD clock
#define SD_DI PMC_IO0_DI // SD data in
#define SD_CS PMC_SD_CS // SD chip select

#define LCD_RX UI_LCD_DISPLAY_RX // LCD serial RX
#define LCD_BAUD UI_LCD_DISPLAY_BAUD // LCD baud rate

#define MAX_PATH 104
#define MAX_ENTRIES 100
#define ENTRY_SIZE 19

// public
int init_fs();
char *navigate_fs();
int create_dir(char *path);
int delete_dir(char *path);
FILE *open_file(char *filename, char *mode);
int close_file(FILE *file_pointer);
size_t write_file(void *ptr, size_t size, size_t count, FILE *file_pointer);
int delete_file(char *filename);
char *get_curr_dir(char *buf, int size);
int set_curr_dir(char *path);
char *path_add_slash(char *path);
bool path_append(char *path, char *more);
char *path_prev_level(char *path);
void up_dir_level();

// private
void sort();
bool compare_swap(int pos);
void empty_dir_entries();
void clear_entry_str(char *entry);
int index_dir_entries(char *dir_path);
void print_dir_entries();
void update_lcd();
void sound_alert();
void update_cursor();
void on_scroll_up_lcd();
void on_scroll_down_lcd();
void on_sel_file_dir();
void clear_selected_entry();
void on_confirm_sel();
void on_reset_root();
void on_up_level();

#endif /* _FILE_SERVICES_H */